unit DataSetSource;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Db, DbTables, ExtCtrls;

type
  TDataSetSource = class(TDataSource)
  private
    fTable: TTable;
    fCompsToDoList: TList;
    fTimer: TTimer;
    procedure SetActive(const Value: Boolean);
    procedure SetDatabaseName(const Value: string);
    procedure SetTableName(const Value: string);
    function GetActive: Boolean;
    function GetDatabaseName: string;
    function GetTableName: string;
  protected
    { Protected declarations }
    procedure TimerProc (Sender: TObject);
  public
    constructor Create (AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Notification (
      AComponent: TComponent; Operation: TOperation); override;
  published
    property DatabaseName: string read GetDatabaseName write SetDatabaseName;
    property TableName: string read GetTableName write SetTableName;
    property Active: Boolean read GetActive write SetActive;
  end;

procedure Register;

implementation

uses
  DynDb;

procedure Register;
begin
  RegisterComponents('FunSide', [TDataSetSource]);
end;

{ TDataSetSource }

constructor TDataSetSource.Create(AOwner: TComponent);
begin
  inherited Create (AOwner);
  fTable := TTable.Create (self);
  fTable.DatabaseName := 'DBDEMOS';
  fTable.TableName := 'BIOLIFE.DB';
  fTable.Name := 'Internal';
  DataSet := fTable;
  fTimer := TTimer.Create (self);
  fTimer.Enabled := False;
  fTimer.OnTimer := TimerProc;
  fCompsToDoList := TList.Create;
end;

destructor TDataSetSource.Destroy;
begin
  fCompsToDoList.Free;
  fTImer.Free;
  inherited Destroy;
end;

function TDataSetSource.GetActive: Boolean;
begin
  if Assigned (fTable) then
    Result := fTable.Active
  else
    Result := False;
end;

function TDataSetSource.GetDatabaseName: string;
begin
  if Assigned (fTable) then
    Result := fTable.DatabaseName
  else
    Result := '';
end;

function TDataSetSource.GetTableName: string;
begin
  if Assigned (fTable) then
    Result := fTable.TableName
  else
    Result := '';
end;

procedure TDataSetSource.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited;

  if Assigned (AComponent) and
    (AComponent <> self) and
    (Operation = opInsert) and
    IsDataAware (AComponent) then
  begin
    fCompsToDoList.Add (AComponent);
    fTimer.Enabled := True;
  end;
end;

procedure TDataSetSource.SetActive(const Value: Boolean);
begin
  fTable.Active := Value;
end;

procedure TDataSetSource.SetDatabaseName(const Value: string);
begin
  fTable.DatabaseName := Value;
end;

procedure TDataSetSource.SetTableName(const Value: string);
begin
  fTable.TableName := Value;
end;

procedure TDataSetSource.TimerProc(Sender: TObject);
var
  Comp: TComponent;
begin
  if fCompsToDoList.Count > 0 then
  begin
    Comp := fCompsToDoList [0];
    // set properties
    if IsDataAware (Comp) then
      ConnectDataSource (Comp, self);
    if HasDataField (Comp) and
        DataSet.Active then
      ConnectDataField (Comp, DataSet.Fields [Random (
          DataSet.FieldCount)].FieldName);
    // remove component from list
    fCompsToDoList.Remove (Comp);
  end;
  if fCompsToDoList.Count = 0 then
    fTimer.Enabled := False;
end;

initialization
  Randomize;
end.
